/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozCellML code.
 *
 * The Initial Developer of the Original Code is
 * The Bioengineering Institute, University of Auckland.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrew Miller <ak.miller@auckland.ac.nz>
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _MOZCELLMLCHANNEL_H
#define _MOZCELLMLCHANNEL_H

#include "mozCellML.h"
#include <nsIChannel.h>
#include <nsIDocumentObserver.h>

#define MOZICELLMLIMPORTCONSUMER_IID_STR "c60af5e4-cbe7-443a-b347-f934609eb2bb"
#define MOZICELLMLIMPORTCONSUMER_IID {0xc60af5e4, 0xcbe7, 0x443a, \
                                      {0xb3, 0x47, 0xf9, 0x34, 0x60, 0x9e, \
                                       0xb2, 0xbb}}

class mozCellMLIImportConsumer
  : public nsISupports
{
public:
  mozCellMLIImportConsumer() {};
  ~mozCellMLIImportConsumer() {};

  NS_DEFINE_STATIC_IID_ACCESSOR(MOZICELLMLIMPORTCONSUMER_IID)
  
  virtual void ImportLoaded(nsIDOMElement*, nsIDOMDocument*) = 0;
};

class mozCellMLChannel
  : public nsIChannel, public nsIDocumentObserver,
    public mozCellMLIImportConsumer
{
public:
  mozCellMLChannel(nsIChannel* aChan);

  NS_DECL_ISUPPORTS
  NS_DECL_NSIREQUEST
  NS_DECL_NSICHANNEL
  NS_DECL_NSIDOCUMENTOBSERVER

  void ImportLoaded(nsIDOMElement*, nsIDOMDocument*);
private:
  ~mozCellMLChannel();
  PRUint32 nRemaining;
  nsCOMPtr<nsIChannel> mChan;
  nsCOMPtr<mozCellML> model;
  nsCOMPtr<nsIDocument> mDoc;
  nsCOMPtr<nsIURI> mURI;
  nsCOMPtr<nsILoadGroup> mLoadGroup;
  nsCOMPtr<nsIMutableArray> mImportRequests;
};

class mozCellMLImportRequest
  : public nsIDocumentObserver
{
public:
  mozCellMLImportRequest(nsIDOMElement*);
  ~mozCellMLImportRequest();

  NS_DECL_ISUPPORTS
  NS_DECL_NSIDOCUMENTOBSERVER

  nsresult StartLoad(nsILoadGroup*, mozCellMLChannel*);
private:
  nsCOMPtr<nsIDOMElement> importNode;
  nsCOMPtr<nsIChannel> mChannel;
  nsCOMPtr<nsIDocument> mDoc;
  nsCOMPtr<mozCellMLIImportConsumer> superChannel;
};
#endif // _MOZCELLMLCHANNEL_H
